************************************************************************
**  ^ROUTINE: WORESR - Reverse communication level routine for
**                     truncated Orthores acceleration. (^)
**
      subroutine zoresr ( ijob , ireq , u , uexact , b , iva , ivql , 
     &   ivqr , iwk , fwk , iparm , fparm , ier )
**
**  ^DESCRIPTION:
**    Orthores iterative method.
**
**  ^AUTHOR:   wdj@beta.lanl.gov
**
**  ^MODIFIED: wdj@lithos.c3.lanl.gov Thu May  6 13:12:38 MDT 1993
**
**  ^ARGUMENTS: see *Subroutine Arguments* below.
**
**  ^REQUIREMENTS:
**    Common Blocks: none
**    Subroutines:   see below.
**
**  ^SIDE_EFFECTS:
**    See the definition of METHRDCLS to see which parameters
**    are modified
**
**  ^ALGORITHM:
**    Truncated Orthores
**
**  ^REFERENCES:
**    David M. Young and Kang **    Conjugate-Gradient Acceleration of Nonsymmetrizable Iterative
**    Methods,'' {\sl Linear Algebra and its Applications}, vol. 34, 1980,
**    pp. 159-194.
**
**  ^DOCUMENTATION:
**
************************************************************************
*
      implicit none
          Include 'fcube.h'
*         Include 'veclib.h'
*
*#    METHRAL        - (^)
      integer ijob
      integer ireq
      integer ier
       integer iwk(*)
       integer iparm(*)
       double complex fwk(*)
       double complex fparm(*)
       double complex u(*)
       double complex uexact(*)
       double complex b(*)
      integer ivql
      integer iva
      integer ivqr
*
        integer itimer
        double precision tc1, tr1, tc2, tr2
      integer iitab
      integer iftab
      integer iwkstg
      integer iclstg
      integer in
      integer is
      integer iretlb
      integer ivitmp, ivotmp
      integer iv1tst, iv2tst
      integer istab1, istab2, istab
      integer ib1,ib2, iu1,iu2
      integer iud
      parameter (iud =0)
      integer ibd
      parameter (ibd =1)
      integer ilbd
      parameter (ilbd =2)
      integer irlbd
      parameter (irlbd=3)
      integer ied
      parameter (ied =4)
      integer ird
      parameter (ird =5)
      integer ilrd
      parameter (ilrd =6)
      integer irlrd
      parameter (irlrd=7)
      double complex urndf
      double complex top
      double complex bot
      parameter (urndf=.1)
      double complex mydot1, mydot2
      external zargck
      external zckstg
      external zwrnd
      external zifppr
      external zitput
      external xtimer
      integer ipme
      integer iphost
      integer log2np
      integer nproc
      integer iom
      character*72 errstr
      external ximini
      external ximal
      external xifre
      external zfmini
      external zfmal
      external zffre
*     integer imv1, imv2        ! temporaries - for matvec
      integer iuvec
      integer ir
      integer ilr
      integer irlr
      integer idots
      integer iv1
      integer iv2
      integer iarlr
      integer ilarlr
      integer ns, nv, nvr, nvrlr, np
      integer i
      double complex rdot
      double complex rnorm
      double complex r0norm
      double complex sig
      double complex sigsum
      double complex uudot, uexdt
*
*
****^^******************************************************************
*     $Modified: spencer@navier.ae.utexas.edu on Thu May 16 11:59:25 1996 $
*     $Id: oresr.fm4,v 1.7 1994/11/22 05:20:42 joubert Exp $
*     $Revision: 1.7 $
*     $Log: oresr.fm4,v $
*     Revision 1.7  1994/11/22  05:20:42  joubert
*     misc changes for release 1.0
*
*     Revision 1.6  1994/10/06  16:24:38  joubert
*     fixed Intel flop count problem; ifparm param name changes
*
*     Revision 1.5  1994/09/28  19:25:08  joubert
*     mods to stats and ntest options
*
*     Revision 1.4  1994/07/06  05:52:48  joubert
*     mods to it meths
*
*     Revision 1.3  1994/07/05  22:21:00  joubert
*     lanczos/orthores acceleration; some cleanup of iterative methods
*
*     Revision 1.2  1994/06/24  18:21:08  joubert
*     t3d version mods; installed iom it meth
*
*     Revision 1.1  1994/06/07  17:40:25  joubert
*     installed orthores iterative method
*
************************************************************************
*
      iitab = (iparm(6))
      iftab = (iparm(6))
      iuvec = (iparm(6))
      ir = (iparm(6))
      ilr = (iparm(6))
      irlr = (iparm(6))
      iv1 = (iparm(6))
      iv2 = (iparm(6))
      idots = (iparm(6))
*
*
      if (ijob .ne. 1) go to 1000
*
      iclstg = 1
      itimer = 0
      in = 0
      iparm(5) = in
      is = 0
*
      iu1 = (iparm(6))
      iu2 = (iparm(6))
      ib1 = (iparm(6))
      ib2 = (iparm(6))
      istab1 = (iparm(6))
      istab2 = (iparm(6))
*
      ipme = mynode ( )
      iphost = myhost ( )
      log2np = nodedim ( )
      nproc = 2**log2np
      if (iparm(27) .eq. 0) iparm(27) = 0
      if (iparm(27) .eq. 0) then
        ier = 0
        if (ijob.eq.1 .or. ijob.eq.2 .or. ijob.eq.0) then
      call ximini ( iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call zfmini ( iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
        endif
        call zargck ( iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
        call xtimer (tc1,tr1,iparm(22),1)
      endif
*
      call xtimer (tc1,tr1,iparm(22),0)
      itimer = 1
*
      if (iparm(27) .eq. 0) then
        call zifppr ( 1 , iparm, fparm, ier)
           if (ier .lt. 0) go to 900
      endif
*
      call ximal (iitab,30, iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call zfmal (iftab,7, iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
*
*
      if (iparm(2) .ge. 3) then
        if (ipme .eq. 0) then
          write (iparm(1),'(/''     Truncated Orthores Method.''/)')
        endif
      endif
*
      ns = iparm(16)
      nv = max(1,ns+1)
*
      call zfmal (iv1,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call zfmal (iv2,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call zfmal (iuvec,iparm(3)*(nv), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call zfmal (ilr,iparm(3)*(nv), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      if (((iparm(13).eq.1).or.(iparm(13).eq.3))) then
        if (((mod(iparm(15)/2,2).eq.1) .or. iparm(12).eq.2 .or. iparm(
     &     12).eq.6)) then
          nvr = nv
      call zfmal (ir,iparm(3)*(nvr), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
        else
          nvr = 1
          ir = iv1
        endif
      else
        nvr = nv
        ir = ilr
      endif
      if (((iparm(13).eq.2).or.(iparm(13).eq.3))) then
        nvrlr = 1
      call zfmal (irlr,iparm(3)*(nvrlr), iparm , fparm , iwk , fwk , 
     &   ier )
           if (ier .lt. 0) go to 900
      else
        nvrlr = nv
        irlr = ilr
      endif
      iarlr = iv1
      if (((iparm(13).eq.1).or.(iparm(13).eq.3))) then
        ilarlr = iv2
      else
        ilarlr = iv1
      endif
      call zfmal (idots,nv, iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
*
      if (iparm(14) .eq. -2 .or. iparm(14) .eq. -1) then
          call zwfil (u,(0d0,0.0d0), iparm , fparm , iwk , fwk , ier )
      endif
*
      if (iparm(14) .eq. 2 ) go to 5500
                 go to 5501
 5500  continue
          call zwrdot (mydot1,b, b, iparm , fparm , iwk , fwk , ier )
        call zwrnd (u, iparm , fparm , iwk , fwk , ier )
      call zfmal (ivitmp,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
          ivqr = ivitmp
          call zwset (fwk(ivqr),u, iparm , fparm , iwk , fwk , ier )
          iva = iv1
       ireq = 3
      iretlb = 2000
      go to 1500
 2000 continue
      call zffre (ivitmp,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
       if (ijob .eq. -1) go to 900
          call zwrdot (mydot2,fwk(iv1), fwk(iv1), iparm , fparm , iwk , 
     &       fwk , ier )
          call zwscl ( u, u, urndf*sqrt(mydot1/mydot2), iparm , fparm , 
     &       iwk , fwk , ier )
                go to 5502
 5501 continue
 5502 continue
*
      if (iparm(14) .eq. 3) go to 5503
                 go to 5504
 5503  continue
      call zfmal (ivitmp,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
          ivqr = ivitmp
          call zwset (fwk(ivqr),u, iparm , fparm , iwk , fwk , ier )
          iva = iv1
       ireq = 3
      iretlb = 2001
      go to 1500
 2001 continue
      call zffre (ivitmp,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
       if (ijob .eq. -1) go to 900
          call zwsub ( fwk(iv1), b, fwk(iv1), iparm , fparm , iwk , fwk 
     &       , ier )
          call zwrdot (mydot1,fwk(iv1), fwk(iv1), iparm , fparm , iwk , 
     &       fwk , ier )
          call zwset (fwk(iv1),u, iparm , fparm , iwk , fwk , ier )
        call zwrnd (u, iparm , fparm , iwk , fwk , ier )
      call zfmal (ivitmp,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
          ivqr = ivitmp
          call zwset (fwk(ivqr),u, iparm , fparm , iwk , fwk , ier )
          iva = iv2
       ireq = 3
      iretlb = 2002
      go to 1500
 2002 continue
      call zffre (ivitmp,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
       if (ijob .eq. -1) go to 900
          call zwrdot (mydot2,fwk(iv2), fwk(iv2), iparm , fparm , iwk , 
     &       fwk , ier )
          call zwsax (u,fwk(iv1), u, -urndf*sqrt(mydot1/mydot2), iparm ,
     &        fparm , iwk , fwk , ier )
                go to 5505
 5504 continue
 5505 continue
          call zwset (fwk(iuvec+iparm(3)*((mod((in),nv)))),u, iparm , 
     &       fparm , iwk , fwk , ier )
*
      if (iparm(21) .eq. 1) go to 5506
                 go to 5507
 5506  continue
      call zfmal (istab1,8, iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call zfmal (istab2,8, iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call zfmal (ib1,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call zfmal (ib2,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call zfmal (iu1,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call zfmal (iu2,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
        istab = istab1
        if (iparm(19) .eq. 1) then
          call zwdot (fwk(istab+iud), uexact, uexact, iparm , fparm , 
     &       iwk , fwk , ier )
        endif
          call zwdot (fwk(istab+ibd ), b, b, iparm , fparm , iwk , fwk ,
     &        ier )
         if (.not. ((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5509
                 go to 5510
 5509  continue
          call zwset (fwk(ib2),b, iparm , fparm , iwk , fwk , ier )
                go to 5511
 5510 continue
      call zfmal (ivitmp,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
          iva = ivitmp
          call zwset (fwk(iva),b, iparm , fparm , iwk , fwk , ier )
          ivql = ib2
       ireq = 5
      iretlb = 2003
      go to 1500
 2003 continue
      call zffre (ivitmp,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
       if (ijob .eq. -1) go to 900
 5511 continue
          call zwdot (fwk(istab+ilbd), fwk(ib2), fwk(ib2), iparm , 
     &       fparm , iwk , fwk , ier )
          call zwset (fwk(iu1),fwk(ib2), iparm , fparm , iwk , fwk , 
     &       ier )
         if (.not. ((iparm(13).eq.2).or.(iparm(13).eq.3))) go to 5512
                 go to 5513
 5512  continue
          call zwset (fwk(iu2),fwk(iu1), iparm , fparm , iwk , fwk , 
     &       ier )
                go to 5514
 5513 continue
          ivql = iu1
          ivqr = iu2
       ireq = 7
      iretlb = 2004
      go to 1500
 2004 continue
       if (ijob .eq. -1) go to 900
 5514 continue
          call zwdot (fwk(istab+irlbd), fwk(iu2), fwk(iu2), iparm , 
     &       fparm , iwk , fwk , ier )
        if (iparm(19) .eq. 1) then
          call zwsub ( fwk(iu1), u, uexact, iparm , fparm , iwk , fwk , 
     &       ier )
          call zwdot (fwk(istab+ied), fwk(iu1), fwk(iu1), iparm , fparm 
     &       , iwk , fwk , ier )
        endif
      call zfmal (ivitmp,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
          ivqr = ivitmp
          call zwset (fwk(ivqr),u, iparm , fparm , iwk , fwk , ier )
          iva = ib1
       ireq = 3
      iretlb = 2005
      go to 1500
 2005 continue
      call zffre (ivitmp,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
       if (ijob .eq. -1) go to 900
          call zwsub ( fwk(ib1), b, fwk(ib1), iparm , fparm , iwk , fwk 
     &       , ier )
          call zwdot (fwk(istab+ird), fwk(ib1), fwk(ib1), iparm , fparm 
     &       , iwk , fwk , ier )
         if (.not. ((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5515
                 go to 5516
 5515  continue
          call zwset (fwk(ib2),fwk(ib1), iparm , fparm , iwk , fwk , 
     &       ier )
                go to 5517
 5516 continue
          iva = ib1
          ivql = ib2
       ireq = 5
      iretlb = 2006
      go to 1500
 2006 continue
       if (ijob .eq. -1) go to 900
 5517 continue
          call zwdot (fwk(istab+ilrd ), fwk(ib2), fwk(ib2), iparm , 
     &       fparm , iwk , fwk , ier )
          call zwset (fwk(iu1),fwk(ib2), iparm , fparm , iwk , fwk , 
     &       ier )
         if (.not. ((iparm(13).eq.2).or.(iparm(13).eq.3))) go to 5518
                 go to 5519
 5518  continue
          call zwset (fwk(iu2),fwk(iu1), iparm , fparm , iwk , fwk , 
     &       ier )
                go to 5520
 5519 continue
          ivql = iu1
          ivqr = iu2
       ireq = 7
      iretlb = 2007
      go to 1500
 2007 continue
       if (ijob .eq. -1) go to 900
 5520 continue
          call zwdot (fwk(istab+irlrd), fwk(iu2), fwk(iu2), iparm , 
     &       fparm , iwk , fwk , ier )
        if (iparm(2) .ge. 4) then
        if (ipme .eq. 0) then
          write (iparm(1),'()')
        endif
        if (ipme .eq. 0) then
          write (iparm(1),'(1x, 60(''*''))')
        endif
        if (ipme .eq. 0) then
          write (iparm(1),'()')
        endif
        if (ipme .eq. 0) then
          write (iparm(1),'(5x,''Initial iteration statistics'')')
        endif
        if (ipme .eq. 0) then
          write (iparm(1),'()')
        endif
        if (iparm(19) .eq. 1) then
        if (ipme .eq. 0) then
          write (iparm(1),'(5x,''2-norm of uexact            '',1pg20.10
     &,2x,1pg20.10)') sqrt(fwk(istab+iud))
        endif
        endif
        if (ipme .eq. 0) then
          write (iparm(1),'(5x,''2-norm of b                 '',1pg20.10
     &,2x,1pg20.10)') sqrt(fwk(istab+ibd))
        endif
        if (ipme .eq. 0) then
          write (iparm(1),'(5x,''2-norm of Ql*b              '',1pg20.10
     &,2x,1pg20.10)') sqrt(fwk(istab+ilbd))
        endif
        if (ipme .eq. 0) then
          write (iparm(1),'(5x,''2-norm of Qr*Ql*b           '',1pg20.10
     &,2x,1pg20.10)') sqrt(fwk(istab+irlbd))
        endif
        if (iparm(19) .eq. 1) then
        if (ipme .eq. 0) then
          write (iparm(1),'(5x,''2-norm of error             '',1pg20.10
     &,2x,1pg20.10)') sqrt(fwk(istab+ied))
        endif
        endif
        if (ipme .eq. 0) then
          write (iparm(1),'(5x,''2-norm of residual          '',1pg20.10
     &,2x,1pg20.10)') sqrt(fwk(istab+ird))
        endif
        if (ipme .eq. 0) then
          write (iparm(1),'(5x,''2-norm of Ql*r              '',1pg20.10
     &,2x,1pg20.10)') sqrt(fwk(istab+ilrd))
        endif
        if (ipme .eq. 0) then
          write (iparm(1),'(5x,''2-norm of Qr*Ql*r           '',1pg20.10
     &,2x,1pg20.10)') sqrt(fwk(istab+irlrd))
        endif
        if (ipme .eq. 0) then
          write (iparm(1),'()')
        endif
        if (ipme .eq. 0) then
          write (iparm(1),'(1x, 60(''*''))')
        endif
        if (ipme .eq. 0) then
          write (iparm(1),'()')
        endif
        endif
      call zffre (iu2,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call zffre (iu1,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call zffre (ib2,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call zffre (ib1,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
                go to 5508
 5507 continue
 5508 continue
*
      if (((iparm(14).eq.0).or.(iparm(14).eq.-2)) .or. iparm(14).eq.-1) 
     &   go to 5521
                 go to 5522
 5521  continue
        if (((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5524
                 go to 5525
 5524  continue
          if (((mod(iparm(15)/2,2).eq.1) .or. iparm(12).eq.2 .or. iparm(
     &       12).eq.6)) go to 5527
                 go to 5528
 5527  continue
          call zwset (fwk(ir+iparm(3)*((mod((in),nvr)))),b, iparm , 
     &       fparm , iwk , fwk , ier )
         if (.not. ((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5530
                 go to 5531
 5530  continue
          call zwset (fwk(ilr+iparm(3)*((mod((in),nv)))),fwk(ir+iparm(3)
     &       *((mod((in),nvr)))), iparm , fparm , iwk , fwk , ier )
                go to 5532
 5531 continue
          iva = ir + iparm(3)*((mod((in),nvr)))
          ivql = ilr + iparm(3)*((mod((in),nv)))
       ireq = 5
      iretlb = 2008
      go to 1500
 2008 continue
       if (ijob .eq. -1) go to 900
 5532 continue
                go to 5529
 5528 continue
         if (.not. ((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5533
                 go to 5534
 5533  continue
          call zwset (fwk(ilr+iparm(3)*((mod((in),nv)))),b, iparm , 
     &       fparm , iwk , fwk , ier )
                go to 5535
 5534 continue
      call zfmal (ivitmp,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
          iva = ivitmp
          call zwset (fwk(iva),b, iparm , fparm , iwk , fwk , ier )
          ivql = ilr + iparm(3)*((mod((in),nv)))
       ireq = 5
      iretlb = 2009
      go to 1500
 2009 continue
      call zffre (ivitmp,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
       if (ijob .eq. -1) go to 900
 5535 continue
 5529 continue
                go to 5526
 5525 continue
          call zwset (fwk(ilr+iparm(3)*((mod((in),nv)))),b, iparm , 
     &       fparm , iwk , fwk , ier )
 5526 continue
                go to 5523
 5522 continue
        if (((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5536
                 go to 5537
 5536  continue
          if (((mod(iparm(15)/2,2).eq.1) .or. iparm(12).eq.2 .or. iparm(
     &       12).eq.6)) go to 5539
                 go to 5540
 5539  continue
          ivqr = iuvec + iparm(3)*((mod((in),nv)))
          iva = ir + iparm(3)*((mod((in),nv)))
       ireq = 3
      iretlb = 2010
      go to 1500
 2010 continue
       if (ijob .eq. -1) go to 900
          call zwsub ( fwk(ir+iparm(3)*((mod((in),nv)))), b, fwk(ir+
     &       iparm(3)*((mod((in),nv)))), iparm , fparm , iwk , fwk , 
     &       ier )
         if (.not. ((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5542
                 go to 5543
 5542  continue
          call zwset (fwk(ilr+iparm(3)*((mod((in),nv)))),fwk(ir+iparm(3)
     &       *((mod((in),nv)))), iparm , fparm , iwk , fwk , ier )
                go to 5544
 5543 continue
          iva = ir + iparm(3)*((mod((in),nv)))
          ivql = ilr + iparm(3)*((mod((in),nv)))
       ireq = 5
      iretlb = 2011
      go to 1500
 2011 continue
       if (ijob .eq. -1) go to 900
 5544 continue
                go to 5541
 5540 continue
          ivqr = iuvec + iparm(3)*((mod((in),nv)))
          iva = iv1
       ireq = 3
      iretlb = 2012
      go to 1500
 2012 continue
       if (ijob .eq. -1) go to 900
          call zwsub ( fwk(iv1), b, fwk(iv1), iparm , fparm , iwk , fwk 
     &       , ier )
         if (.not. ((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5545
                 go to 5546
 5545  continue
          call zwset (fwk(ilr+iparm(3)*((mod((in),nv)))),fwk(iv1), 
     &       iparm , fparm , iwk , fwk , ier )
                go to 5547
 5546 continue
          iva = iv1
          ivql = ilr + iparm(3)*((mod((in),nv)))
       ireq = 5
      iretlb = 2013
      go to 1500
 2013 continue
       if (ijob .eq. -1) go to 900
 5547 continue
 5541 continue
                go to 5538
 5537 continue
          ivqr = iuvec + iparm(3)*((mod((in),nv)))
          iva = ilr + iparm(3)*((mod((in),nv)))
       ireq = 3
      iretlb = 2014
      go to 1500
 2014 continue
       if (ijob .eq. -1) go to 900
          call zwsub ( fwk(ilr+iparm(3)*((mod((in),nv)))), b, fwk(ilr+
     &       iparm(3)*((mod((in),nv)))), iparm , fparm , iwk , fwk , 
     &       ier )
 5538 continue
 5523 continue
      if (((iparm(13).eq.2).or.(iparm(13).eq.3))) go to 5548
                 go to 5549
 5548  continue
         if (.not. ((iparm(13).eq.2).or.(iparm(13).eq.3))) go to 5551
                 go to 5552
 5551  continue
          call zwset (fwk(irlr+iparm(3)*((mod((in),nvrlr)))),fwk(ilr+
     &       iparm(3)*((mod((in),nv)))), iparm , fparm , iwk , fwk , 
     &       ier )
                go to 5553
 5552 continue
          ivql = ilr + iparm(3)*((mod((in),nv)))
          ivqr = irlr + iparm(3)*((mod((in),nvrlr)))
       ireq = 7
      iretlb = 2015
      go to 1500
 2015 continue
       if (ijob .eq. -1) go to 900
 5553 continue
                go to 5550
 5549 continue
 5550 continue
*
          call zwrdot (rdot,fwk(ilr+iparm(3)*((mod((in),nv)))), fwk(ilr+
     &       iparm(3)*((mod((in),nv)))), iparm , fparm , iwk , fwk , 
     &       ier )
      rnorm = sqrt(abs(rdot))
      r0norm = rnorm
      fwk((idots+mod((in),nv))) = rdot
*
*---------------------------iteration loop------------------------------
*
  100 continue
*
        if (iparm(2) .ge. 5) then
        if (ipme .eq. 0) then
          write (iparm(1),500) 'rnorm               ', rnorm
        endif
        endif
*
        if (iparm(12).eq.-3) go to 5554
                 go to 5555
 5554  continue
          if ((mod(iparm(15)/1,2).eq.1)) then
          call zwset (u,fwk(iuvec+iparm(3)*((mod((in),nv)))), iparm , 
     &       fparm , iwk , fwk , ier )
          endif
          if ((mod(iparm(15)/2,2).eq.1)) then
          iva = ir + iparm(3)*((mod((in),nvr)))
          endif
          if ((mod(iparm(15)/4,2).eq.1)) then
          ivql = ilr + iparm(3)*((mod((in),nv)))
          endif
          if ((mod(iparm(15)/8,2).eq.1)) then
          ivqr = irlr + iparm(3)*((mod((in),nvrlr)))
          endif
          ireq = 9
      iretlb = 2016
      go to 1500
 2016 continue
          if ((mod(iparm(15)/8,2).eq.1)) then
          endif
          if ((mod(iparm(15)/4,2).eq.1)) then
          endif
          if ((mod(iparm(15)/2,2).eq.1)) then
          endif
          if ((mod(iparm(15)/1,2).eq.1)) then
          endif
          if (ier .ge. 0) then
            call zitput ( in , is , iparm , fparm , iwk , fwk , ier )
          endif
          if (ijob .eq. -1) then
            if (iparm(2).ge.3 .and. iparm(27).eq.0) then
        if (ipme .eq. 0) then
          write (iparm(1),'(/a/)') ' User-requested termination of itera
     &tive method.'
        endif
            endif
            go to 900
          endif
                go to 5556
 5555 continue
 5556 continue
        if (iparm(12) .gt. 1) go to 5557
                 go to 5558
 5557  continue
          if (iparm(12) .eq. 1 ) then
          call zwset (u,fwk(iuvec+iparm(3)*((mod((in),nv)))), iparm , 
     &       fparm , iwk , fwk , ier )
            if (iparm(5) .eq. 0) then
              if (iparm(19) .eq. 0) then
                ier = -5
                call xersho ( ier, 'zoresr' , iparm , '19 and 12 incompa
     &tible' )
                go to 900
              endif
          call zwdot (bot, uexact, uexact, iparm , fparm , iwk , fwk , 
     &       ier )
              top = bot
            else
      call zfmal (iv1tst,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
          call zwsub ( fwk(iv1tst), u, uexact, iparm , fparm , iwk , 
     &       fwk , ier )
          call zwdot (top, fwk(iv1tst), fwk(iv1tst), iparm , fparm , 
     &       iwk , fwk , ier )
      call zffre (iv1tst,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
            endif
          endif
          if (iparm(12) .eq. 2 ) then
            if (iparm(5) .eq. 0) then
          call zwdot (bot, b, b, iparm , fparm , iwk , fwk , ier )
              top = bot
            else
          call zwdot (top, fwk(ir+iparm(3)*((mod((in),nvr)))), fwk(ir+
     &       iparm(3)*((mod((in),nvr)))), iparm , fparm , iwk , fwk , 
     &       ier )
            endif
          endif
          if (iparm(12) .eq. 3) go to 5560
                 go to 5561
 5560  continue
            if (iparm(5) .eq. 0) go to 5563
                 go to 5564
 5563  continue
              if (((iparm(14).eq.0).or.(iparm(14).eq.-2))) go to 5566
                 go to 5567
 5566  continue
          call zwdot (bot, fwk(ilr+iparm(3)*((mod((in),nv)))), fwk(ilr+
     &       iparm(3)*((mod((in),nv)))), iparm , fparm , iwk , fwk , 
     &       ier )
                go to 5568
 5567 continue
      call zfmal (iv1tst,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
         if (.not. ((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5569
                 go to 5570
 5569  continue
          call zwset (fwk(iv1tst),b, iparm , fparm , iwk , fwk , ier )
                go to 5571
 5570 continue
      call zfmal (ivitmp,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
          iva = ivitmp
          call zwset (fwk(iva),b, iparm , fparm , iwk , fwk , ier )
          ivql = iv1tst
       ireq = 5
      iretlb = 2017
      go to 1500
 2017 continue
      call zffre (ivitmp,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
       if (ijob .eq. -1) go to 900
 5571 continue
          call zwdot (bot, fwk(iv1tst), fwk(iv1tst), iparm , fparm , 
     &       iwk , fwk , ier )
      call zffre (iv1tst,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
 5568 continue
              top = bot
                go to 5565
 5564 continue
          call zwdot (top, fwk(ilr+iparm(3)*((mod((in),nv)))), fwk(ilr+
     &       iparm(3)*((mod((in),nv)))), iparm , fparm , iwk , fwk , 
     &       ier )
 5565 continue
                go to 5562
 5561 continue
 5562 continue
          if (iparm(12) .eq. 4) go to 5572
                 go to 5573
 5572  continue
            if (iparm(5) .eq. 0) go to 5575
                 go to 5576
 5575  continue
              if (((iparm(14).eq.0).or.(iparm(14).eq.-2))) go to 5578
                 go to 5579
 5578  continue
          call zwdot (bot, fwk(irlr+iparm(3)*((mod((in),nvrlr)))), fwk(
     &       irlr+iparm(3)*((mod((in),nvrlr)))), iparm , fparm , iwk , 
     &       fwk , ier )
                go to 5580
 5579 continue
      call zfmal (iv1tst,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call zfmal (iv2tst,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
         if (.not. ((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5581
                 go to 5582
 5581  continue
          call zwset (fwk(iv1tst),b, iparm , fparm , iwk , fwk , ier )
                go to 5583
 5582 continue
      call zfmal (ivitmp,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
          iva = ivitmp
          call zwset (fwk(iva),b, iparm , fparm , iwk , fwk , ier )
          ivql = iv1tst
       ireq = 5
      iretlb = 2018
      go to 1500
 2018 continue
      call zffre (ivitmp,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
       if (ijob .eq. -1) go to 900
 5583 continue
         if (.not. ((iparm(13).eq.2).or.(iparm(13).eq.3))) go to 5584
                 go to 5585
 5584  continue
          call zwset (fwk(iv2tst),fwk(iv1tst), iparm , fparm , iwk , 
     &       fwk , ier )
                go to 5586
 5585 continue
          ivql = iv1tst
          ivqr = iv2tst
       ireq = 7
      iretlb = 2019
      go to 1500
 2019 continue
       if (ijob .eq. -1) go to 900
 5586 continue
          call zwdot (bot, fwk(iv2tst), fwk(iv2tst), iparm , fparm , 
     &       iwk , fwk , ier )
      call zffre (iv2tst,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call zffre (iv1tst,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
 5580 continue
              top = bot
                go to 5577
 5576 continue
          call zwdot (top, fwk(irlr+iparm(3)*((mod((in),nvrlr)))), fwk(
     &       irlr+iparm(3)*((mod((in),nvrlr)))), iparm , fparm , iwk , 
     &       fwk , ier )
 5577 continue
                go to 5574
 5573 continue
 5574 continue
          if (iparm(12) .eq. 1 ) then
          call zwset (u,fwk(iuvec+iparm(3)*((mod((in),nv)))), iparm , 
     &       fparm , iwk , fwk , ier )
            if (iparm(19) .eq. 0) then
                ier = -5
                call xersho ( ier, 'zoresr' , iparm , '19 and 12 incompa
     &tible' )
                go to 900
            endif
            if (((iparm(14).eq.0).or.(iparm(14).eq.-2))) then
          call zwdot (top, uexact, uexact, iparm , fparm , iwk , fwk , 
     &       ier )
            else
      call zfmal (iv1tst,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
          call zwsub ( fwk(iv1tst), u, uexact, iparm , fparm , iwk , 
     &       fwk , ier )
          call zwdot (top, fwk(iv1tst), fwk(iv1tst), iparm , fparm , 
     &       iwk , fwk , ier )
      call zffre (iv1tst,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
            endif
            if (iparm(5) .eq. 0) bot = top
          endif
          if (iparm(12) .eq. 6) then
          call zwdot (top, fwk(ir+iparm(3)*((mod((in),nvr)))), fwk(ir+
     &       iparm(3)*((mod((in),nvr)))), iparm , fparm , iwk , fwk , 
     &       ier )
            if (iparm(5) .eq. 0) bot = top
          endif
          if (iparm(12) .eq. 7) then
          call zwdot (top, fwk(ilr+iparm(3)*((mod((in),nv)))), fwk(ilr+
     &       iparm(3)*((mod((in),nv)))), iparm , fparm , iwk , fwk , 
     &       ier )
            if (iparm(5) .eq. 0) bot = top
          endif
          if (iparm(12) .eq. 8) then
          call zwdot (top, fwk(irlr+iparm(3)*((mod((in),nvrlr)))), fwk(
     &       irlr+iparm(3)*((mod((in),nvrlr)))), iparm , fparm , iwk , 
     &       fwk , ier )
            if (iparm(5) .eq. 0) bot = top
          endif
          if (dble(top) .lt. dble((0d0,0.0d0))) then
                ier = -10
                call xersho ( ier, 'zoresr' , iparm , 'Attempt to take s
     &qrt of negative number' )
                go to 900
          endif
          if (dble(bot) .le. dble((0d0,0.0d0))) then
                ier = -10
                call xersho ( ier, 'zoresr' , iparm , 'Attempt to divide
     & by zero' )
                go to 900
          endif
          fparm(5) = sqrt(dble(top)/dble(bot))
          call zitput ( in , is , iparm , fparm , iwk , fwk , ier )
          if (dble(fparm(5)) .le. dble(fparm(4))) go to 2500
                go to 5559
 5558 continue
 5559 continue
*
        if (iparm(12) .eq. -1) then
          if (r0norm .eq. (0d0,0.0d0)) then
                ier = -10
                call xersho ( ier, 'zoresr' , iparm , 'Attempt to divide
     & by zero' )
                go to 900
          endif
          fparm(5) = rnorm/r0norm
          call zitput ( in , is , iparm , fparm , iwk , fwk , ier )
          if (dble(fparm(5)) .le. dble(fparm(4))) go to 2500
        endif
*
        if (iparm(12) .eq. -2) then
          call zwsub ( fwk(iv1), fwk(iuvec+iparm(3)*((mod((in),nv)))), 
     &       uexact, iparm , fparm , iwk , fwk , ier )
          call zwrdot (uudot,fwk(iv1), fwk(iv1), iparm , fparm , iwk , 
     &       fwk , ier )
          call zwrdot (uexdt,uexact, uexact, iparm , fparm , iwk , fwk ,
     &        ier )
          if (dble(uudot) .lt. dble((0d0,0.0d0))) then
                ier = -10
                call xersho ( ier, 'zoresr' , iparm , 'Attempt to take s
     &qrt of negative number' )
                go to 900
          endif
          if (dble(uexdt) .le. dble((0d0,0.0d0))) then
                ier = -10
                call xersho ( ier, 'zoresr' , iparm , 'Attempt to divide
     & by zero' )
                go to 900
          endif
          fparm(5) = sqrt(dble(uudot)/dble(uexdt))
          call zitput ( in , is , iparm , fparm , iwk , fwk , ier )
          if (dble(fparm(5)) .le. dble(fparm(4))) go to 2500
        endif
*
        if (in .ge. iparm(4)) then
                ier = 2
                call xersho ( ier, 'zoresr' , iparm , ' ' )
          go to 900
        endif
*
        call zckstg ( iclstg, iwkstg, iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
*
*-----------------------proceed with iteration--------------------------
*
        np = min(in+1,ns+1)
          ivqr = irlr + iparm(3)*((mod((in),nvrlr)))
          iva = iarlr
       ireq = 3
      iretlb = 2020
      go to 1500
 2020 continue
       if (ijob .eq. -1) go to 900
        if (((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5587
                 go to 5588
 5587  continue
         if (.not. ((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5590
                 go to 5591
 5590  continue
          call zwset (fwk(ilarlr),fwk(iarlr), iparm , fparm , iwk , fwk 
     &       , ier )
                go to 5592
 5591 continue
          iva = iarlr
          ivql = ilarlr
       ireq = 5
      iretlb = 2021
      go to 1500
 2021 continue
       if (ijob .eq. -1) go to 900
 5592 continue
                go to 5589
 5588 continue
 5589 continue
        i = in - np + 1
          call zwcdot (sig, fwk(ilr+iparm(3)*((mod((i),nv)))), fwk(
     &       ilarlr), iparm(20), iparm , fparm , iwk , fwk , ier )
        if (fwk((idots+mod((i),nv))) .eq. (0d0,0.0d0)) then
                ier = -6
                call xersho ( ier, 'zoresr' , iparm , ' ' )
                go to 900
        endif
        sig = sig / fwk((idots+mod((i),nv)))
          call zwsax (fwk(iuvec+iparm(3)*((mod((in+1),nv)))),fwk(irlr+
     &       iparm(3)*((mod((in),nvrlr)))), fwk(iuvec+iparm(3)*((mod((i)
     &       ,nv)))), sig, iparm , fparm , iwk , fwk , ier )
          call zwsax (fwk(ilr+iparm(3)*((mod((in+1),nv)))),fwk(ilarlr), 
     &       fwk(ilr+iparm(3)*((mod((i),nv)))), -sig, iparm , fparm , 
     &       iwk , fwk , ier )
        if (((iparm(13).eq.1).or.(iparm(13).eq.3)) .and. ((mod(iparm(15)
     &     /2,2).eq.1) .or. iparm(12).eq.2 .or. iparm(12).eq.6)) then
          call zwsax (fwk(ir+iparm(3)*((mod((in+1),nvr)))),fwk(iarlr), 
     &       fwk(ir+iparm(3)*((mod((i),nvr)))), -sig, iparm , fparm , 
     &       iwk , fwk , ier )
        endif
        sigsum = sig
*
        do 8511 i = in-np+2, in
          call zwcdot (sig, fwk(ilr+iparm(3)*((mod((i),nv)))), fwk(
     &       ilarlr), iparm(20), iparm , fparm , iwk , fwk , ier )
          if (fwk((idots+mod((i),nv))) .eq. (0d0,0.0d0)) then
                ier = -6
                call xersho ( ier, 'zoresr' , iparm , ' ' )
                go to 900
          endif
          sig = sig / fwk((idots+mod((i),nv)))
          call zwsax (fwk(iuvec+iparm(3)*((mod((in+1),nv)))),fwk(iuvec+
     &       iparm(3)*((mod((in+1),nv)))), fwk(iuvec+iparm(3)*((mod((i),
     &       nv)))), sig, iparm , fparm , iwk , fwk , ier )
          call zwsax (fwk(ilr+iparm(3)*((mod((in+1),nv)))),fwk(ilr+
     &       iparm(3)*((mod((in+1),nv)))), fwk(ilr+iparm(3)*((mod((i),
     &       nv)))), -sig, iparm , fparm , iwk , fwk , ier )
          if (((iparm(13).eq.1).or.(iparm(13).eq.3)) .and. ((mod(iparm(
     &       15)/2,2).eq.1) .or. iparm(12).eq.2 .or. iparm(12).eq.6)) 
     &       then
          call zwsax (fwk(ir+iparm(3)*((mod((in+1),nvr)))),fwk(ir+iparm(
     &       3)*((mod((in+1),nvr)))), fwk(ir+iparm(3)*((mod((i),nvr)))),
     &        -sig, iparm , fparm , iwk , fwk , ier )
          endif
          sigsum = sigsum + sig
 8511   continue
*
        if (sigsum .eq. (0d0,0.0d0)) then
                ier = -6
                call xersho ( ier, 'zoresr' , iparm , ' ' )
                go to 900
        endif
          call zwscl ( fwk(iuvec+iparm(3)*((mod((in+1),nv)))), fwk(
     &       iuvec+iparm(3)*((mod((in+1),nv)))), (1d0,0.0d0)/sigsum, 
     &       iparm , fparm , iwk , fwk , ier )
          call zwscl ( fwk(ilr+iparm(3)*((mod((in+1),nv)))), fwk(ilr+
     &       iparm(3)*((mod((in+1),nv)))), -(1d0,0.0d0)/sigsum, iparm , 
     &       fparm , iwk , fwk , ier )
        if (((iparm(13).eq.1).or.(iparm(13).eq.3)) .and. ((mod(iparm(15)
     &     /2,2).eq.1) .or. iparm(12).eq.2 .or. iparm(12).eq.6)) then
          call zwscl ( fwk(ir+iparm(3)*((mod((in+1),nvr)))), fwk(ir+
     &       iparm(3)*((mod((in+1),nvr)))), -(1d0,0.0d0)/sigsum, iparm ,
     &        fparm , iwk , fwk , ier )
        endif
*
        if (((iparm(13).eq.2).or.(iparm(13).eq.3))) go to 5593
                 go to 5594
 5593  continue
         if (.not. ((iparm(13).eq.2).or.(iparm(13).eq.3))) go to 5596
                 go to 5597
 5596  continue
          call zwset (fwk(irlr+iparm(3)*((mod((in+1),nvrlr)))),fwk(ilr+
     &       iparm(3)*((mod((in+1),nv)))), iparm , fparm , iwk , fwk , 
     &       ier )
                go to 5598
 5597 continue
          ivql = ilr + iparm(3)*((mod((in+1),nv)))
          ivqr = irlr + iparm(3)*((mod((in+1),nvrlr)))
       ireq = 7
      iretlb = 2022
      go to 1500
 2022 continue
       if (ijob .eq. -1) go to 900
 5598 continue
                go to 5595
 5594 continue
 5595 continue
*
          call zwrdot (rdot,fwk(ilr+iparm(3)*((mod((in+1),nv)))), fwk(
     &       ilr+iparm(3)*((mod((in+1),nv)))), iparm , fparm , iwk , 
     &       fwk , ier )
        fwk((idots+mod((in+1),nv))) = rdot
        rnorm = sqrt(abs(rdot))
*
        in = in + 1
        iparm(5) = in
        is = is + 1
*
      go to 100
*
 2500 continue
      if (iparm(2) .ge. 3) then
        if (ipme .eq. 0) then
          write (iparm(1),'(/a,i7,a/)') '     Truncated Orthores Method 
     &has converged in ', iparm(5), ' iterations.'
        endif
      endif
*
*-------------------------------terminate-------------------------------
*
  900 continue
*
*     ---set u last time----------
      if (.not.(iparm(12).eq.-3.and.(mod(iparm(15)/1,2).eq.1))) then
          call zwset (u,fwk(iuvec+iparm(3)*((mod((in),nv)))), iparm , 
     &       fparm , iwk , fwk , ier )
      endif
*
      if (iclstg .ne. 1) then
        call zckstg (-1, iwkstg, iparm , fparm , iwk , fwk , ier )
      endif
*
      if (iparm(21).eq.1 .and. ier.ge.0) go to 5599
                 go to 5600
 5599  continue
      call zfmal (ib1,iparm(3), iparm , fparm , iwk , fwk , ier )
      call zfmal (ib2,iparm(3), iparm , fparm , iwk , fwk , ier )
      call zfmal (iu1,iparm(3), iparm , fparm , iwk , fwk , ier )
      call zfmal (iu2,iparm(3), iparm , fparm , iwk , fwk , ier )
        istab = istab2
        if (iparm(19) .eq. 1) then
          call zwdot (fwk(istab+iud), uexact, uexact, iparm , fparm , 
     &       iwk , fwk , ier )
        endif
          call zwdot (fwk(istab+ibd ), b, b, iparm , fparm , iwk , fwk ,
     &        ier )
         if (.not. ((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5602
                 go to 5603
 5602  continue
          call zwset (fwk(ib2),b, iparm , fparm , iwk , fwk , ier )
                go to 5604
 5603 continue
      call zfmal (ivitmp,iparm(3), iparm , fparm , iwk , fwk , ier )
          iva = ivitmp
          call zwset (fwk(iva),b, iparm , fparm , iwk , fwk , ier )
          ivql = ib2
       ireq = 5
      iretlb = 2023
      go to 1500
 2023 continue
      call zffre (ivitmp,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
       if (ijob .eq. -1) go to 900
 5604 continue
          call zwdot (fwk(istab+ilbd), fwk(ib2), fwk(ib2), iparm , 
     &       fparm , iwk , fwk , ier )
          call zwset (fwk(iu1),fwk(ib2), iparm , fparm , iwk , fwk , 
     &       ier )
         if (.not. ((iparm(13).eq.2).or.(iparm(13).eq.3))) go to 5605
                 go to 5606
 5605  continue
          call zwset (fwk(iu2),fwk(iu1), iparm , fparm , iwk , fwk , 
     &       ier )
                go to 5607
 5606 continue
          ivql = iu1
          ivqr = iu2
       ireq = 7
      iretlb = 2024
      go to 1500
 2024 continue
       if (ijob .eq. -1) go to 900
 5607 continue
          call zwdot (fwk(istab+irlbd), fwk(iu2), fwk(iu2), iparm , 
     &       fparm , iwk , fwk , ier )
        if (iparm(19) .eq. 1) then
          call zwsub ( fwk(iu1), u, uexact, iparm , fparm , iwk , fwk , 
     &       ier )
          call zwdot (fwk(istab+ied), fwk(iu1), fwk(iu1), iparm , fparm 
     &       , iwk , fwk , ier )
        endif
      call zfmal (ivitmp,iparm(3), iparm , fparm , iwk , fwk , ier )
          ivqr = ivitmp
          call zwset (fwk(ivqr),u, iparm , fparm , iwk , fwk , ier )
          iva = ib1
       ireq = 3
      iretlb = 2025
      go to 1500
 2025 continue
      call zffre (ivitmp,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
       if (ijob .eq. -1) go to 900
          call zwsub ( fwk(ib1), b, fwk(ib1), iparm , fparm , iwk , fwk 
     &       , ier )
          call zwdot (fwk(istab+ird), fwk(ib1), fwk(ib1), iparm , fparm 
     &       , iwk , fwk , ier )
         if (.not. ((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5608
                 go to 5609
 5608  continue
          call zwset (fwk(ib2),fwk(ib1), iparm , fparm , iwk , fwk , 
     &       ier )
                go to 5610
 5609 continue
          iva = ib1
          ivql = ib2
       ireq = 5
      iretlb = 2026
      go to 1500
 2026 continue
       if (ijob .eq. -1) go to 900
 5610 continue
          call zwdot (fwk(istab+ilrd ), fwk(ib2), fwk(ib2), iparm , 
     &       fparm , iwk , fwk , ier )
          call zwset (fwk(iu1),fwk(ib2), iparm , fparm , iwk , fwk , 
     &       ier )
         if (.not. ((iparm(13).eq.2).or.(iparm(13).eq.3))) go to 5611
                 go to 5612
 5611  continue
          call zwset (fwk(iu2),fwk(iu1), iparm , fparm , iwk , fwk , 
     &       ier )
                go to 5613
 5612 continue
          ivql = iu1
          ivqr = iu2
       ireq = 7
      iretlb = 2027
      go to 1500
 2027 continue
       if (ijob .eq. -1) go to 900
 5613 continue
          call zwdot (fwk(istab+irlrd), fwk(iu2), fwk(iu2), iparm , 
     &       fparm , iwk , fwk , ier )
        if (iparm(19) .eq. 1) then
          if (fwk(istab1+ied) .eq. (0d0,0.0d0)) then
                ier = 4
                call xersho ( ier, 'zoresr' , iparm , 'Value of RELERR m
     &ay be inaccurate' )
          else
            fparm(8) = sqrt(fwk(istab2+ied)/fwk(istab1+ied))
          endif
        endif
        if (fwk(istab1+ird) .eq. (0d0,0.0d0)) then
                ier = 4
                call xersho ( ier, 'zoresr' , iparm , 'Value of RELRSD m
     &ay be inaccurate' )
        else
          fparm(7) = sqrt(fwk(istab2+ird)/fwk(istab1+ird))
        endif
        if (iparm(2) .ge. 4) then
        if (ipme .eq. 0) then
          write (iparm(1),'()')
        endif
        if (ipme .eq. 0) then
          write (iparm(1),'(1x, 60(''*''))')
        endif
        if (ipme .eq. 0) then
          write (iparm(1),'()')
        endif
        if (ipme .eq. 0) then
          write (iparm(1),'(5x,''Final iteration statistics'')')
        endif
        if (ipme .eq. 0) then
          write (iparm(1),'()')
        endif
        if (iparm(19) .eq. 1) then
        if (ipme .eq. 0) then
          write (iparm(1),'(5x,''2-norm of uexact            '',1pg20.10
     &,2x,1pg20.10)') sqrt(fwk(istab+iud))
        endif
        endif
        if (ipme .eq. 0) then
          write (iparm(1),'(5x,''2-norm of b                 '',1pg20.10
     &,2x,1pg20.10)') sqrt(fwk(istab+ibd))
        endif
        if (ipme .eq. 0) then
          write (iparm(1),'(5x,''2-norm of Ql*b              '',1pg20.10
     &,2x,1pg20.10)') sqrt(fwk(istab+ilbd))
        endif
        if (ipme .eq. 0) then
          write (iparm(1),'(5x,''2-norm of Qr*Ql*b           '',1pg20.10
     &,2x,1pg20.10)') sqrt(fwk(istab+irlbd))
        endif
        if (iparm(19) .eq. 1) then
        if (ipme .eq. 0) then
          write (iparm(1),'(5x,''2-norm of error             '',1pg20.10
     &,2x,1pg20.10)') sqrt(fwk(istab+ied))
        endif
        endif
        if (ipme .eq. 0) then
          write (iparm(1),'(5x,''2-norm of residual          '',1pg20.10
     &,2x,1pg20.10)') sqrt(fwk(istab+ird))
        endif
        if (ipme .eq. 0) then
          write (iparm(1),'(5x,''2-norm of Ql*r              '',1pg20.10
     &,2x,1pg20.10)') sqrt(fwk(istab+ilrd))
        endif
        if (ipme .eq. 0) then
          write (iparm(1),'(5x,''2-norm of Qr*Ql*r           '',1pg20.10
     &,2x,1pg20.10)') sqrt(fwk(istab+irlrd))
        endif
        if (iparm(19) .eq. 1) then
        if (ipme .eq. 0) then
          write (iparm(1),'(5x,''Relative 2-norm of error    '',1pg20.10
     &,2x,1pg20.10)') fparm(8)
        endif
        endif
        if (ipme .eq. 0) then
          write (iparm(1),'(5x,''Relative 2-norm of residual '',1pg20.10
     &,2x,1pg20.10)') fparm(7)
        endif
        if (ipme .eq. 0) then
          write (iparm(1),'()')
        endif
        if (ipme .eq. 0) then
          write (iparm(1),'(1x, 60(''*''))')
        endif
        if (ipme .eq. 0) then
          write (iparm(1),'()')
        endif
        endif
      call zffre (iu2,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
      call zffre (iu1,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
      call zffre (ib2,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
      call zffre (ib1,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
      call zffre (istab2,8, iparm , fparm , iwk , fwk , ier )
      call zffre (istab1,8, iparm , fparm , iwk , fwk , ier )
                go to 5601
 5600 continue
 5601 continue
*
      call zffre (idots,nv, iparm , fparm , iwk , fwk , ier )
      if (((iparm(13).eq.2).or.(iparm(13).eq.3))) then
      call zffre (irlr,iparm(3)*(nvrlr), iparm , fparm , iwk , fwk , 
     &   ier )
      endif
      if (((iparm(13).eq.1).or.(iparm(13).eq.3))) then
        if (((mod(iparm(15)/2,2).eq.1) .or. iparm(12).eq.2 .or. iparm(
     &     12).eq.6)) then
      call zffre (ir,iparm(3)*(nvr), iparm , fparm , iwk , fwk , ier )
        endif
      endif
      call zffre (ilr,iparm(3)*(nv), iparm , fparm , iwk , fwk , ier )
      call zffre (iuvec,iparm(3)*(nv), iparm , fparm , iwk , fwk , ier )
      call zffre (iv2,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
      call zffre (iv1,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
*
*
*
*
      call zffre (iftab,7, iparm , fparm , iwk , fwk , ier )
      call xifre (iitab,30, iparm , fparm , iwk , fwk , ier )
*
      if (itimer .eq. 1) then
        call xtimer (tc2,tr2,iparm(22),0)
        fparm(1) = fparm(1) + (tc2-tc1)
        fparm(2) = fparm(2) + (tr2-tr1)
      endif
*
  910 continue
      if (iparm(27) .eq. 0) then
        call zifppr ( -1 , iparm, fparm, ier )
      endif
*
      if (iparm(27) .eq. 0) iparm(27) = 0
      iparm(11) = (iparm(6))
      ireq = -1
      return
*
*-------------------------------brancher--------------------------------
*
*
 1000 continue
*
      ipme = mynode ( )
      iphost = myhost ( )
      log2np = nodedim ( )
      nproc = 2**log2np
      iitab = iparm(11)
      if (iitab.eq.(iparm(6)) .and. ijob.eq.-1) go to 910
*
      if (iparm(11) .eq. (iparm(6))) then
                ier = -4
                call xersho ( ier, 'zoresr' , iparm , 'Values of ijob an
     &d iparm(iptr) incompatible' )
      endif
*
      call xtimer (tc1,tr1,iparm(22),0)
*
        iitab = iwk(iitab+0)
        iftab = iwk(iitab+1)
        istab = iwk(iitab+2)
        iwkstg = iwk(iitab+3)
        iclstg = iwk(iitab+4)
        in = iwk(iitab+5)
        is = iwk(iitab+6)
        itimer = iwk(iitab+7)
        iretlb = iwk(iitab+8)
        ivitmp = iwk(iitab+9)
        ivotmp = iwk(iitab+10)
        iv1tst = iwk(iitab+11)
        iv2tst = iwk(iitab+12)
        istab1 = iwk(iitab+13)
        istab2 = iwk(iitab+14)
        iuvec = iwk(iitab+15)
        ir = iwk(iitab+16)
        ilr = iwk(iitab+17)
        irlr = iwk(iitab+18)
        idots = iwk(iitab+19)
        iv1 = iwk(iitab+20)
        iv2 = iwk(iitab+21)
        iarlr = iwk(iitab+22)
        ilarlr = iwk(iitab+23)
        ns = iwk(iitab+24)
        nv = iwk(iitab+25)
        nvr = iwk(iitab+26)
        nvrlr = iwk(iitab+27)
        np = iwk(iitab+28)
        i = iwk(iitab+29)
        top = fwk(iftab+0)
        bot = fwk(iftab+1)
        rdot = fwk(iftab+2)
        rnorm = fwk(iftab+3)
        r0norm = fwk(iftab+4)
        sig = fwk(iftab+5)
        sigsum = fwk(iftab+6)
*
      if (ijob.ne.3 .and. ijob.ne.-1) then
                ier = -4
                call xersho ( ier, 'zoresr' , iparm , 'Argument ijob inv
     &alid' )
      endif
*
      go to (2000,2001,2002,2003,2004,2005,2006,2007,2008,2009,2010,
     &   2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022,
     &   2023,2024,2025,2026,2027), iretlb - 2000 + 1
*
*---------------------------request of caller---------------------------
*
*
 1500 continue
      iparm(11) = iitab
        iwk(iitab+0) = iitab
        iwk(iitab+1) = iftab
        iwk(iitab+2) = istab
        iwk(iitab+3) = iwkstg
        iwk(iitab+4) = iclstg
        iwk(iitab+5) = in
        iwk(iitab+6) = is
        iwk(iitab+7) = itimer
        iwk(iitab+8) = iretlb
        iwk(iitab+9) = ivitmp
        iwk(iitab+10) = ivotmp
        iwk(iitab+11) = iv1tst
        iwk(iitab+12) = iv2tst
        iwk(iitab+13) = istab1
        iwk(iitab+14) = istab2
        iwk(iitab+15) = iuvec
        iwk(iitab+16) = ir
        iwk(iitab+17) = ilr
        iwk(iitab+18) = irlr
        iwk(iitab+19) = idots
        iwk(iitab+20) = iv1
        iwk(iitab+21) = iv2
        iwk(iitab+22) = iarlr
        iwk(iitab+23) = ilarlr
        iwk(iitab+24) = ns
        iwk(iitab+25) = nv
        iwk(iitab+26) = nvr
        iwk(iitab+27) = nvrlr
        iwk(iitab+28) = np
        iwk(iitab+29) = i
        fwk(iftab+0) = top
        fwk(iftab+1) = bot
        fwk(iftab+2) = rdot
        fwk(iftab+3) = rnorm
        fwk(iftab+4) = r0norm
        fwk(iftab+5) = sig
        fwk(iftab+6) = sigsum
      call xtimer (tc2,tr2,iparm(22),0)
      fparm(1) = fparm(1) + (tc2-tc1)
      fparm(2) = fparm(2) + (tr2-tr1)
      return
*
*
*--------------------------------end------------------------------------
*
  500 format ( 1x, a, 1pg20.10,2x,1pg20.10 )
*
      end
